<?php

require __DIR__ . '/libs/initpro-kassa-php-sdk/autoload.php';

use Initpro\KassaSdk\Client;
use Initpro\KassaSdk\GroupManager;
use Initpro\KassaSdk\Check;
use Initpro\KassaSdk\Payment;
use Initpro\KassaSdk\Position;
use Initpro\KassaSdk\Vat;
use Initpro\KassaSdk\Exception\ClientException;
use Initpro\KassaSdk\Exception\SdkException;


/**
 * Implements hook_commerce_payment_order_paid_in_full().
 */
function commerce_initpro_kassa_commerce_payment_order_paid_in_full($order, $transaction) {
    initpro_kassa_fiscalize($order, $transaction);
}

function get_amount($value, $get_value = True) {
    $amount = $value->amount;
    if ($get_value) {
        $amount = $amount->value();
    }
    return (float) number_format($amount / 100, 2, '.', '');
}

function get_quantity($value) {
    return (float) number_format($value->quantity->value(), 1);
}

function initpro_kassa_fiscalize($order, $transaction = NULL, $op = 'sell', $is_refund = false) {

    if (empty($order->order_id)) {
        return;
    }

    $wrapper = entity_metadata_wrapper('commerce_order', $order->order_id);

    $payment_method = $is_refund ? Check::INTENT_SELL_RETURN : Check::INTENT_SELL;

    $uniqid = uniqid(strval($order->order_id), true);

    $check = new Check($uniqid,
                       $wrapper->mail->value(), 
                       $payment_method, 
                       (int) variable_get('commerce_initpro_kassa_tax_system', '1'));
    
    $check->setShouldPrint(variable_get('commerce_initpro_kassa_should_print'));

    foreach ($wrapper->commerce_line_items as $key => $line_item_wrapper) {

        $line_item_type = $line_item_wrapper->getBundle();
        $item_sum = get_amount($line_item_wrapper->commerce_total);
        $item_price = get_amount($line_item_wrapper->commerce_unit_price);
        $item_quantity = get_quantity($line_item_wrapper);

        if (in_array($line_item_type, commerce_product_line_item_types())) {
            $item_name = $line_item_wrapper->commerce_product->value()->title;
        } else {
            $item_name = $line_item_wrapper->line_item_label->value();
        }

        $check->addPosition(new Position($item_name, $item_price, $item_quantity, $item_sum, 0, new Vat(Vat::RATE_NO)));
    }

    if ($transaction) {
        $payment = new Payment(Payment::TYPE_CARD, round(get_amount($transaction, false), 2));
        $check->addPayment($payment);
    }
    else {
        $payment = new Payment(Payment::TYPE_CARD, round(get_amount($wrapper->commerce_order_total), 2));
        $check->addPayment($payment);
    }

    $client = new Client(variable_get('commerce_initpro_kassa_shop_id'), 
                         variable_get('commerce_initpro_kassa_secret_key'));
    $client->setHost(variable_get('commerce_initpro_kassa_api_url', COMMERCE_initpro_KASSA_API_URL));
    
    $manager = new GroupManager($client);
    $manager->registerGroup('default',  variable_get('commerce_initpro_kassa_queue_id'));
    $manager->setDefaultGroup('default');

    $response = $error_message = NULL;
    try {
        watchdog('commerce_initpro_kassa', drupal_json_encode($check->asArray()), array(), WATCHDOG_NOTICE, $link = NULL);
        $response = $manager->putCheck($check);
    } catch (SdkException $e) {
        $error_message = $e->getMessage();
        watchdog('commerce_initpro_kassa',  $error_message, array(), WATCHDOG_ERROR, $link = NULL);
    }

    // Report in the report
    $insert_fields = array(
        'created_at' => time(),
        'order_id' => $order->order_id,
        'operation_id' => isset($response) ? $uniqid : NULL,
        'status' => isset($error_message) ? COMMERCE_initpro_KASSA_REPORT_STATUS_ERROR : 'pending',
        'result' => $is_refund ? 'refund' : NULL,
        'error' => $error_message
    );
    db_insert('commerce_initpro_kassa_reports')->fields($insert_fields)->execute();
    
    return $error_message === NULL;
}

function commerce_fiscalize_form($form, &$form_state, $order) {
    $form = array();
    $form_state['order'] = $order;

    $form['submit'] = array(
		'#type' => 'submit',
		'#value' => t('Фискализировать'),
	);

    return $form;
}

function commerce_fiscalize_form_submit($form, &$form_state) {

    $db_pending = db_query("SELECT * FROM commerce_initpro_kassa_reports WHERE order_id = " . $form_state['order']->order_id . " AND `status` != 'error' ORDER BY id LIMIT 1")->fetchAll();
    $db_cancel = db_query("SELECT * FROM commerce_initpro_kassa_reports WHERE order_id = " . $form_state['order']->order_id . " AND `result` = 'refund' ORDER BY id LIMIT 1")->fetchAll();

    if (empty($db_pending) || ($form_state['order']->status == 'canceled' && empty($db_cancel)) ){
        $order = commerce_order_load($form_state['order']->order_id);

        $is_refund = $form_state['order']->status == 'canceled'? true : false;

        if (initpro_kassa_fiscalize($order, NULL, 'sell', $is_refund)) {
            drupal_set_message('Заказ отправлен на фискализацию в сервис initpro Касса');
        } else {
            drupal_set_message('При отправке на фискализацию в сервис initpro Касса произошла ошибка', 'error');
        }
        watchdog('commerce_initpro_kassa', 'Заказ #%order_id был отправлен на фискализацию в систему ИНИТПРО Касса вручную', array('%order_id' => $order->order_id), WATCHDOG_NOTICE, $link = NULL);
    }else{
        if ($form_state['order']->status != 'canceled'){
            drupal_set_message('Данный заказ уже был отправлен на фискализацию', 'error');
        }else{
            drupal_set_message('Данный заказ уже был отправлен на фискализацию как отмененный', 'error');
        }
    }
}

function commerce_fiscalize_ui_order_tab($order) {

    $header = array(
        array('data' => '#', 'field' => 'id'),
        array('data' => 'Дата создания задачи', 'field' => 'created_at'),
        array('data' => 'Дата обновления задачи', 'field' => 'updated_at'),
        array('data' => 'Номер операции в очереди', 'field' => 'operation_id'),
        array('data' => 'Статус задачи', 'field' => 'status'),
    );

    $select = db_select('commerce_initpro_kassa_reports', 'reports');

    $select->fields('reports', array('id', 'order_id', 'created_at', 'updated_at', 'operation_id', 'status'))
           ->condition('order_id', $order->order_number)
           ->orderBy('id', 'desc');

    $db_result = $select->execute();

    $rows = array();

    if ($db_result->rowCount() > 0) {
        $table = '<p><strong>Список задач для фискализации заказа:</strong></p>';
        foreach ($db_result as $row) {
            $rows[] = array(
                'id' => initpro_kassa_report_detail_link($row->id),
                'created_at' => initpro_kassa_display_date($row->created_at),
                'updated_at' => initpro_kassa_display_date($row->updated_at),
                'operation_id' => $row->operation_id,
                'status' => $row->status,
            );
        }
        $table .= theme('table', array('header' => $header, 'rows' => $rows));
    } else {
        $table = theme('status_messages', drupal_set_message('Для заказа нет отправленных задач на фискализацию в сервис ИНИТПРО Касса', 'warning'));
    }

    $form = drupal_get_form('commerce_fiscalize_form', $order);

    return $table . '<br/>' . drupal_render($form); 
}
